@@ -410,11 +410,17 @@ rxi_Listen(osi_socket sock)
 int
 rxi_Recvmsg(osi_socket socket, struct msghdr *msg_p, int flags)
 {
-#if defined(HAVE_LINUX_ERRQUEUE_H) && defined(ADAPT_PMTU)
-    while((rxi_HandleSocketError(socket)) > 0)
-	;
+    int code;
+    code = recvmsg(socket, msg_p, flags);
+
+#ifdef AFS_RXERRQ_ENV
+    if (code < 0) {
+	while((rxi_HandleSocketError(socket)) > 0)
+	    ;
+    }
 #endif
-    return recvmsg(socket, msg_p, flags);
+
+    return code;
 }
 
 /*
@@ -427,6 +433,13 @@ rxi_Sendmsg(osi_socket socket, struct msghdr *msg_p, int flags)
     fd_set *sfds = (fd_set *) 0;
     while (sendmsg(socket, msg_p, flags) == -1) {
 	int err;
+
+#ifdef AFS_NT40_ENV
+	err = WSAGetLastError();
+#else
+	err = errno;
+#endif
+
 	if (rx_stats_active)
 	    rx_atomic_inc(&rx_stats.sendSelects);
 
@@ -438,31 +451,26 @@ rxi_Sendmsg(osi_socket socket, struct msghdr *msg_p, int flags)
 	    }
 	    FD_SET(socket, sfds);
 	}
-#if defined(HAVE_LINUX_ERRQUEUE_H) && defined(ADAPT_PMTU)
+#ifdef AFS_RXERRQ_ENV
 	while((rxi_HandleSocketError(socket)) > 0)
 	  ;
 #endif
 #ifdef AFS_NT40_ENV
-	if (WSAGetLastError())
+	if (err)
 #elif defined(AFS_LINUX22_ENV)
 	/* linux unfortunately returns ECONNREFUSED if the target port
 	 * is no longer in use */
 	/* and EAGAIN if a UDP checksum is incorrect */
-	if (errno != EWOULDBLOCK && errno != ENOBUFS && errno != ECONNREFUSED
-	    && errno != EAGAIN)
+	if (err != EWOULDBLOCK && err != ENOBUFS && err != ECONNREFUSED
+	    && err != EAGAIN)
 #else
-	if (errno != EWOULDBLOCK && errno != ENOBUFS)
+	if (err != EWOULDBLOCK && err != ENOBUFS)
 #endif
 	{
 	    (osi_Msg "rx failed to send packet: ");
 	    perror("rx_sendmsg");
-#ifndef AFS_NT40_ENV
-            if (errno > 0)
-              return -errno;
-#else
-            if (WSAGetLastError() > 0)
-              return -WSAGetLastError();
-#endif
+            if (err > 0)
+              return -err;
 	    return -1;
 	}
 	while ((err = select(
